<?php namespace EscapeWork\Manager\Providers;

use EscapeWork\Manager\Facades\UserAuth;
use EscapeWork\Manager\Models\RoleAction;
use EscapeWork\Manager\Permission;
use Illuminate\Support\ServiceProvider;
use Illuminate\Foundation\AliasLoader;
use Auth;
use App;

class ManagerServiceProvider extends ServiceProvider
{

    /**
     * Indicates if loading of the provider is deferred.
     *
     * @var bool
     */
    protected $defer = false;

    /**
     * Bootstrap the application events.
     *
     * @return void
     */
    public function boot()
    {
        require_once(__DIR__ . '/../../../routes.php');
    }

    /**
     * Register the service provider.
     *
     * @return void
     */
    public function register()
    {
        $this->package('escapework/manager', 'manager', realpath(__DIR__ . '/../../../'));

        $this->registerPermission();
        $this->bindRepositories();
        $this->callProviders();
        $this->createAliases();
    }

    /**
     * Registering the permission check class
     */
    protected function registerPermission()
    {
        $this->app['escapework.manager.permission'] = $this->app->share(function($app)
        {
            $user = UserAuth::user();

            $permission = App::make('EscapeWork\Manager\Permission', array($user, new RoleAction));
            $permission->fetchAllowedActions()
                       ->formatAllowedActionsArray();

            return $permission;
        });
    }

    /**
     * Calling the providers needed
     */
    protected function callProviders()
    {
        $this->app->register('EscapeWork\Manager\Providers\UserServiceProvider');

        # escapework/msg
        $this->app->register('EscapeWork\Msg\Laravel\MsgServiceProvider');
    }

    /**
     * Binding the repositories
     */
    protected function bindRepositories()
    {
        $this->app->bind('UserRepository', 'EscapeWork\Manager\Models\User');
        $this->app->bind('UserReminderRepository', 'EscapeWork\Manager\Models\UserReminder');
        $this->app->bind('RoleRepository', 'EscapeWork\Manager\Models\Role');
        $this->app->bind('RoleActionRepository', 'EscapeWork\Manager\Models\RoleAction');
        $this->app->bind('UserRoleRepository', 'EscapeWork\Manager\Models\UserRole');
        $this->app->bind('PageRepository', 'EscapeWork\Manager\Models\Page');
        $this->app->bind('PageActionRepository', 'EscapeWork\Manager\Models\PageAction');
        $this->app->bind('EscapeWork\Manager\Permission', function($app, $parameters)
        {
            return new Permission($parameters[0], $parameters[1]);
        });
    }

    /**
     * Criando os alias
     */
    protected function createAliases()
    {
        $this->app->booting(function()
        {
            $loader = AliasLoader::getInstance();

            $loader->alias('Check', 'EscapeWork\Manager\Check');
            $loader->alias('Eloquent', 'EscapeWork\LaravelHelpers\Eloquent');
            $loader->alias('Msg', 'EscapeWork\Msg\Msg');
        });
    }

    /**
     * Get the services provided by the provider.
     *
     * @return array
     */
    public function provides()
    {
        return array('escapework.manager.permission');
    }
}